	SUBROUTINE CPB_CRPS ( iscr, mxt, mxd, ndpth, npts, itrc, dtrace,
     +			      rmiss, calp, ncalp, trcdat,
     +                        trcmin, trcmax, iret )
C************************************************************************
C* CPB_CRPS								*
C*									*
C* This subroutine computes the Continuous Ranked Probability Score     *    
C* (CRPS) and its decomposition components as formulated by Hersbach    *
C* (2000).                                                              *
C*									*
C* Reference:                                                           *
C*                                                                      *
C* Hersbach, H., 2000:  Decomposition of the continuous ranked          *
C*     probability score for ensemble prediction systems.  WEA.         *
C*     FORECASTING, 15, 559-570.                                        *
C*									*
C* CPB_CRPS ( ISCR, MXT, MXD, NDPTH, NPTS, ITRC, DTRACE, RMISS, 	*
C*	      CALP, NCALP, TRCDAT, TRCMIN, TRCMAX, IRET )		*
C*									*
C* Input parameters:							*
C*	ISCR		INTEGER		Requested parameter index:      *
C*                                        1 = CRPS                      *
C*                                        2 = Potential CRPS  (CRPSpot) *
C*                                        3 = Reliability component     *
C*                                        4 = Resolution component      *
C*                                        5 = Uncertainty component     *
C*                                        6 = CRPS skill vs uncertainty *
C*	MXT		INTEGER		Max # of traces in DTRACE	*
C*	MXD		INTEGER		Max # of values/point in DTRACE	*
C*	NDPTH		INTEGER		Number of values/point		*
C*	NPTS		INTEGER		Number of points in the trace	*
C*	ITRC		INTEGER		Trace #				*
C*	DTRACE (MXT,MXD,						*
C*		NPTS)	REAL*8		Trace values			*
C*	RMISS		REAL		Missing value for output	*
C*      CALP (NCALP)    REAL            Calibration cumulative probs    *
C*      NCALP           INTEGER         Number of cumulative probs      *
C*									*
C*									*
C* Output parameters:							*
C*	TRCDAT (NPTS)	REAL*8		Output trace			*
C*	TRCMIN		REAL		Minimum output trace value	*
C*	TRCMAX		REAL		Maximum output trace value	*
C*	IRET		INTEGER		Return code			*
C*				 	  0 = normal return 		*
C*                                       -1 = too few calibration probs *
C*                                       -2 = ISCR is not valid         *
C**									*
C* Log:									*
C* K. Brill/IMSG     20160630   Created                                 *
C* K. Brill/IMSG     20160720   Added CPRS skill versus uncertainty     *
C************************************************************************
        PARAMETER       (MXAB=32)
C*
	REAL*8		dtrace (mxt,mxd,*), trcdat (*)
        REAL            calp (*)
C*
	REAL*8		gbar (MXAB), obar (MXAB), ubar
        REAL*8          cprb (MXAB), alfa, beta, rnttl, sum
	LOGICAL		ok
C*-----------------------------------------------------------------------
	iret = 0
C*
        nab = ( ndpth - 4 ) / 2
        IF ( ncalp .lt. (nab-1) ) THEN
            iret = -1
            RETURN
        END IF
C
C*      Load the calibration cumulative probability array:
C
        cprb (1) = 0.0
        cprb (nab) = 1.0
        jj = 0
        DO ii = 2, (nab-1)
            jj = jj + 1
            cprb (ii) = calp (jj)
        END DO
C*
	trcmin = 1.1E30
	trcmax = -1.1E30
	DO ip = 1, npts
	    ok = .true.
	    iok = 0
	    DO WHILE  ( iok .lt. ndpth .and. ok )
		iok = iok + 1
		ok = ( dtrace (itrc,iok,ip) .gt. -1.E30 )
	    END DO
	    rnttl  = dtrace (itrc,1,ip)
            IF ( ok .and. rnttl .gt. 0.0 .and. iscr .eq. 5 ) THEN
                ubar = dtrace (itrc,ndpth,ip) / rnttl
                trcdat (ip) = ubar
	    ELSE IF ( ok .and. rnttl .gt. 0.0 ) THEN
                indx = 2 * nab + 2
                obar (1) = dtrace (itrc,indx,ip) / rnttl
                indx = indx + 1
                obar (nab) = dtrace (itrc,indx,ip) / rnttl
                indx = indx + 1
                ubar = dtrace (itrc,indx,ip) / rnttl
C*              First beta value:
                indx = nab + 2
                beta = dtrace (itrc,indx,ip) / rnttl
                IF ( obar (1) .gt. 0.0 ) THEN
                    gbar (1) = beta / obar (1)
                ELSE
                    gbar (1) = 0.0
                    obar (1) = 0.0
                END IF
C*              Last alpha value:
                indx = nab + 1
                alfa = dtrace (itrc,indx,ip) / rnttl
                IF ( obar (nab) .lt. 1.0 ) THEN
                    gbar (nab) = alfa / ( 1. - obar (nab) )
                ELSE
                    gbar (nab) = 0.0
                    obar (nab) = 1.0
                END IF
C
C*              Compute the interior means for g and o:
C
                indxa = 3
                indxb = nab + 3
		DO ii = 2, (nab-1)
                    alfa = dtrace (itrc,indxa,ip) / rnttl
                    beta = dtrace (itrc,indxb,ip) / rnttl
                    gbar (ii) = alfa + beta
                    IF ( gbar (ii) .gt. 0 ) THEN
                        obar (ii) = beta / gbar (ii)
                    ELSE
                        obar (ii) = 0.0
                    END IF
                    indxa = indxa + 1
                    indxb = indxb + 1
                END DO
C
C*              1 = CRPS -- Hersbach eq. 35                       *
C*              2 = Potential CRPS  -- Hersbach eq. 37            *
C*              3 = Reliability component -- Hersbach eq. 36      *
C*              4 = Resolution component -- Hersbach eq. 38       *
C*              5 = Uncertainty component estimate assigned above *
C*              6 = CRPS skill versus uncertainty                 *
C
                IF ( iscr .eq. 1 ) THEN
                    sum = 0.0
                    DO i = 1, nab
                        psqrd = cprb (i) * cprb (i)
                        cpsqd = ( 1. - cprb (i) ) * ( 1. - cprb (i) )
                        sum = sum + gbar (i) * ( ( 1. - obar (i) ) *
     +                          psqrd + obar (i) * cpsqd )
                    END DO
                    trcdat (ip) = sum
                ELSE IF ( iscr .eq. 2 .or. iscr .eq. 4 ) THEN
                    sum = 0.0
                    DO i = 1, nab
                        sum = sum + gbar (i) *  ( 1. - obar (i) ) *
     +                          obar (i)
                    END DO
                    IF ( iscr .eq. 2 ) THEN
                        trcdat (ip) = sum
                    ELSE
                        trcdat (ip) = ubar - sum
                    END IF
                ELSE IF ( iscr .eq. 3 ) THEN
                    sum = 0.0
                    DO i = 1, nab
                        sum = sum + gbar (i) *  ( obar (i) - cprb (i) ) 
     +                        * ( obar (i) - cprb (i) )
                    END DO
                    trcdat (ip) = sum
                ELSE IF ( iscr .eq. 6 ) THEN
C
C*                  CRPS skill versus uncertainty.
C*                  Perfect value is 1.
C
                    sum = 0.0
                    DO i = 1, nab
                        psqrd = cprb (i) * cprb (i)
                        cpsqd = ( 1. - cprb (i) ) * ( 1. - cprb (i) )
                        sum = sum + gbar (i) * ( ( 1. - obar (i) ) *
     +                          psqrd + obar (i) * cpsqd )
                    END DO
                    IF ( ubar .gt. 0.0 ) THEN
                        trcdat (ip) = ( ubar - sum ) / ubar
                    ELSE
                        trcdat (ip) = rmiss
                    END IF
                ELSE
                    iret = -2
                    trcdat (ip)  = rmiss
                END IF
                IF ( iret .eq. 0 ) THEN
		    IF ( trcdat (ip) .lt. trcmin ) trcmin = trcdat (ip)
		    IF ( trcdat (ip) .gt. trcmax ) trcmax = trcdat (ip)
		END IF
	    ELSE
		trcdat (ip) = rmiss
	    END IF
	END DO
C*
	RETURN
	END
