	SUBROUTINE PLT_IXAX ( axis, cplbl, xplbl, nplbl,
     +			      rmin, rmax, iptype, iret )
C************************************************************************
C* PLT_IXAX								*
C*									*
C* This subroutine processes the x axis control parameter.		*
C*									*
C* An array of axis label point coordinate values are stored.  The	*
C* frequencies with respect to the elements in the array of values	*
C* for the plotting of labels, grid lines and tick marks are also	*
C* stored.  If any of these frequency values is missing, the		*
C* corresponding stored value is set to default values.  Plotting	*
C* begins  with the first element.					*
C* 									*
C* The graphics control file entry is formatted as follows:		*
C*									*
C*  XAXIS = start/stop/increment/labfrq;glnfrq;ticfrq			*
C*									*
C*  XAXIS = t_start:t_stop:increment/labfrq;glnfrq;ticfrq		*
C*									*
C*   or									*
C*									*
C*  XAXIS = start/stop/value1;value2;...;valueN/labfrq;glnfrq;ticfrq	*
C*									*
C* In the latter case, the increment specification has been replaced	*
C* with a list of values.  Failure to specify START and STOP will	*
C* result in default values determined on the basis of RMIN and RMAX.	*
C* A positive increment will generate values divisible by the 		*
C* increment.  A negative increment will generate values incremented	*
C* from START using the absolute value of the increment.		*
C*									*
C* The second case is used for a time axis.  In this case, t_start	*
C* and t_stop must be given in the form YYYYMMDDHH.  The increment is	*
C* in days (including decimal fraction thereof) unless Y or M is used	*
C* as described below.  Note the colon separator distinguishes this	*
C* case from the others.						*
C*									*
C* For a time axis, the increment value may be followed by Y, M, or D	*
C* (default) to denote years, months, or days, respectively.  If the	*
C* increment is negative, the absolute value is used.  If only Y, M,	*
C* or D appears in the increment position, then the increment is	*
C* assumed to be one.							*
C*									*
C* PLT_IXAX  ( AXIS, CPLBL, XPLBL, NPLBL, RMIN, RMAX, IPTYPE, IRET )	*
C*									*
C* Input parameters:							*
C*	AXIS		CHAR*		Input for axis			*
C*	CPLBL (NPLBL)	CHAR*		Character labels (time stamps)	*
C*	XPLBL (NPLBL)	REAL		Label positions			*
C*	NPLBL		INTEGER		Number of labels		*
C*	RMIN            REAL            Data minimum			*
C*	RMAX            REAL            Data maximum			*
C*	IPTYPE		INTEGER		Type of plot			*
C*					  0 - print trace statistics	*
C*					  1 - time series		*
C*					  2 - scatter plot		*
C*					  3 - skill score vs thresholds *
C*					  4 - categorical bins		*
C*					  5 - histogram			*
C*					  6 - ROC curve			*
C*					  7 - attributes diagram	*
C*                                        8 - Quantile RELI diagram     *
C*									*
C* Output parameters:							*
C*	IRET		INTEGER		Return code			*
C*					  0 = normal return		*
C**									*
C* Log:									*
C* K. Brill/EMC		 1/98   Created from IN_AXIS 			*
C* K. Brill/EMC		 4/98	Clean up / bug fixes			*
C* K. Brill/EMC		10/98	Month, year increments for t axis;	*
C*				Date-time label bounds			*
C* K. Brill/HPC		 9/03	Document IPTYPE=6			*
C* K. Brill/HPC      20070125	Document IPTYPE=7			*
C* K. Brill/IMSG     20160718	Document IPTYPE=8			*
C************************************************************************
	INCLUDE		'plt.cmn'
	PARAMETER	( RMISSD = -9999., IMISSD = -9999 )
	PARAMETER	( RDIFFD =  0.1    )
C!			  Missing value fuzziness

C*
	CHARACTER*(*)	cplbl (*), axis
	REAL		xplbl (*)
C*
	CHARACTER*256	caxis
	CHARACTER*72    substr (4)
	INTEGER         ihold (3)
	LOGICAL ERMISS
C*
	ERMISS (xxxx) = ( ABS ( xxxx - RMISSD ) .lt. RDIFFD )
C------------------------------------------------------------------------
	iret = 0
	rint = RMISSD
	dmin = rmin
	dmax = rmax
	ilfdef = 1
	igfdef = 0
	itfdef = 1
	CALL PLT_ITAX ( axis, cplbl, xplbl, nplbl, caxis, iret )
	IF ( iret .ne. 0 ) RETURN
C
C*	Set DMIN and DMAX under appropriate circumstances.
C
	IF ( dmin .eq. dmax ) THEN
	  IF ( dmin .eq. 0. ) THEN
	    dmin = -.5
	    dmax =  .5
	  ELSE
	    test = abs ( dmin )
	    ipow = ALOG10 ( test ) - 1
	    add  = .5 * ( 10. ** ipow )
	    dmax = dmax + add
	    dmin = dmin - add
	  END IF
	END IF
C
C*      Break AXIS into four substrings.
C
	CALL ST_CLST ( caxis, '/', ' ', 4, substr, nst, ier )
C
C*	Break the fourth substring into 3 frequency integers.  If the
C*      values are missing, set them to the input default values.
C
	CALL ST_ILST ( substr (4), ';', IMISSD, 3, ihold, nm, ier )
	DO i = 1, 2
	  IF ( ihold (i) .lt. 0 ) ihold (i) = IMISSD
	END DO
	IF  ( ihold (1) .ne. IMISSD )	THEN
            ilbfrx = ihold (1)
	ELSE
	    ilbfrx = ilfdef
        END IF
	IF  ( ihold (2) .ne. IMISSD )   THEN
            iglfrx = ihold (2)
        ELSE
            iglfrx = igfdef
	END IF
        IF  ( ihold (3) .ne. IMISSD )	THEN
            itmfrx = ihold (3)
	    IF ( itmfrx .lt. 0 ) THEN
		itmfrx = 1
		itmbtw = ABS ( ihold (3) )
	    ELSE
		itmbtw = 0
	    END IF
        ELSE
	    itmfrx = itfdef
	    itmbtw = 0
        END IF
C
C*      Decode START and STOP from first and second substrings.
C
	CALL ST_CRNM ( substr (1), start, ier )
	CALL ST_CRNM ( substr (2), stop,  ier )
C
C*      If START and STOP are still missing, establish defaults.
C
	IF ( ERMISS ( start ) ) THEN
	    IF ( iptype .ne. 5 ) THEN
		start = dmin - .0375 * ABS ( dmax - dmin )
	    ELSE
		start = rmin
	    END IF
	END IF
	IF ( ERMISS ( stop ) ) THEN
	    IF ( iptype .ne. 5 ) THEN
		stop = dmax + .0375 * ABS ( dmax - dmin )
	    ELSE
		stop = rmax
	    END IF
	END IF
C
C*	At this point START and STOP are defined and not missing.
C
C*	Set time axis unit and increment.
C
	CALL ST_RMBL ( substr (3), substr(3), lnn, ier )
	CALL ST_LCUC ( substr (3), substr(3), ier )
	iyr = INDEX ( substr (3), 'Y' )
	imn = INDEX ( substr (3), 'M' )
	idy = INDEX ( substr (3), 'D' )
	IF ( iyr .ne. 0 ) THEN
	    iq = iyr
	    unttax = 'Y'
	ELSE IF ( imn .ne. 0 ) THEN
	    iq = imn
	    unttax = 'M'
	ELSE IF ( idy .ne. 0 ) THEN
	    iq = idy
	    unttax = 'D'
	ELSE
	    iq = 0
	    unttax = 'D'
	END IF
	inctax = 1
	IF ( iq .gt. 1 ) substr (3) = substr (3) (1:iq-1)
	IF ( iq .eq. 1 ) substr (3) = '1'
C
C*      Look at the third substring to see if it contains a list of
C*      values or a single number giving the increment.
C
	CALL ST_RLST ( substr (3), ';', RMISSD, MAXVAL,
     +                 xaxval, nval, ier )
	IF ( .not. ERMISS ( xaxval (1) ) ) THEN
	    rint = xaxval (1)
	    inctax = NINT ( ABS ( rint ) )
	END IF
C
C*	Generate axis labels.
C
C*	If NVAL is not 1, do nothing; otherwise, generate the values
C*      using the start value, the increment and the stop value.
C
	IF ( nval .gt. 1 ) THEN
C
C*          Do nothing.
C
	ELSE IF ( xaxval (1) .eq. 0.0 ) THEN
	    nval = 2
	    xaxval (1) = start
	    xaxval (2) = stop
C*
	ELSE IF ( xaxval (1) .gt. 0 .or. ERMISS ( xaxval (1) ) ) THEN
C
C*	    Set a convenient default interval.
C
	    IF ( ERMISS (rint) ) THEN
		delx = ABS ( stop - start )
		alpha = delx / 10.0
		ipwr = 0
		DO WHILE ( ( alpha .lt. 1 .or. alpha .gt. 10 ) .and.
     +			     alpha .ne. 0.0 )
		    IF ( alpha .lt. 1. ) THEN
			ipwr = ipwr - 1
			alpha = alpha * 10.
		    ELSE IF ( alpha .gt. 10 ) THEN
			ipwr = ipwr + 1
			alpha = alpha / 10.
		    END IF
		END DO
		dy = FLOAT ( NINT ( alpha ) )
		rint = dy * 10. ** ipwr
	    END IF
	    inctax = NINT ( rint )
C
C*	    Set values that are divisible by the interval.  First,
C*	    find multiplier to use for RINT to compute minimum value.
C
	    k   =  start / rint
	    xstrt = FLOAT (k) * rint
	    dq = rint
	    IF ( start .gt. stop .and. dq .gt. 0.0 ) dq = -dq
	    diff = ABS ( stop - start )
	    xaxval (1) = xstrt
	    nval = 1
	    chck = ABS ( xaxval (nval) + dq - start )
	    DO WHILE ( nval .lt. MAXVAL .and. chck .le. diff )
		nval = nval + 1
		xaxval ( nval ) = xaxval ( nval - 1 ) + dq 
		IF ( ABS ( xaxval (nval) - 0.0 ) .lt.
     +			.05 * ABS ( dq ) ) xaxval (nval) = 0.0
		chck = ABS ( xaxval (nval) + dq - start )
	    END DO

C*
	ELSE IF ( xaxval (1) .lt. 0 ) THEN
C
C*	    Set values by incrementing from START.
C
	    dq = -xaxval (1)
	    IF ( start .gt. stop .and. dq .gt. 0.0 ) dq = -dq
	    diff = ABS ( stop - start )
	    xaxval (1) = start
	    nval = 1
	    chck = ABS ( xaxval (nval) + dq - start )
	    DO WHILE ( nval .lt. MAXVAL .and. chck .le. diff )
		nval = nval + 1
		xaxval ( nval ) = xaxval ( nval - 1 ) + dq 
		IF ( ABS ( xaxval (nval) - 0.0 ) .lt.
     +			.05 * ABS ( dq ) ) xaxval (nval) = 0.0
		chck = ABS ( xaxval (nval) + dq - start )
	    END DO
	END IF
	xaxbeg = start
	xaxend = stop
	nxaxvl = nval
	RETURN
	END
