	SUBROUTINE PLT_ZSIG ( mp, mt, ntrc, trc, npts, prmnam,
     +			      clbl1, clbl2, iret )
C************************************************************************
C* PLT_ZSIG								*
C*									*
C* This subroutine writes out statistical significance analysis results *
C* for two traces.							*
C*									*
C* This statistical significance analysis tests the difference from	*
C* zero of the differences of paired values between two traces.  It is	*
C* based on sections 5.2.3 and 5.2.4 of "Statistical Methods in the	*
C* Atmospheric Sciences" 2nd Edition (2006) by D.S. Wilks.  The treat-	*
C* ment of the data as paired follows the recommendations of Hamill	*
C* (1999).  The null hypothesis is that the mean of differences is 	*
C* zero, i.e., the two traces are the same.  The alternative hypothesis *
C* is obtained diagnostically by examining the arithmetic sign of the	*
C* mean of the differences.  The lag-1 serial correlation of the	*
C* differences of the paired values is used to inflate the variance	*
C* estimate for the mean of differences.  The probability estimates for *
C* the decisions are derived from a Gaussian distribution with mean 0	*
C* and standard deviation 1 for the serial correlation adjusted Z test	*
C* value.  The raw unadjusted Z value, adjusted Z value, number of	*
C* pairs, and serial correlation are written out in addition to the	*
C* decision making information.  Results are written to a file named	*
C* fvs_sig_test1.txt.							*
C*									*
C* Ideally, this significance analysis should be applied to two matched *
C* time series of data.  The number of points in the time series should *
C* be at least 28.  For example, a time series of FHO values for the	*
C* one-inch precipitation threshold from an operational model and a     *
C* parallel version could be used to test the significance of the	*
C* difference in the equitable threat score between the two.  Following	*
C* Hamill (1999), such comparisons require that the contingency table   *
C* associated with the FHO values be well populated for each pair of	*
C* threat scores.  Also, it must be shown that no statistically		*
C* significant difference in bias (F/O) exists between the two time	*
C* series.  If the former condition is not met, then a resampling	*
C* technique will give a more reliable estimate of statistical		*
C* significance (Hamill, 1999).  If the latter condition is not met,	*
C* a bias correction of some sort must be applied.			*
C*									*
C* If the traces have less than 28 matched pairs, then the "t" test	*
C* should be used on the Z value.  The "t" distribution has more	*
C* probability in the tails.  Alternatively, for short time series,	*
C* a lower test level for the probability of wrongly accepting the null	*
C* hypothesis could be required if the Gaussian distribution is used.   *
C*									*
C*                           References					*
C*									*
C* Hamill, T.M., 1999:  Hypothesis tests for evaluating numerical	*
C*    precipitation forecasts.  WEA. FORCASTING, 14, 155--167.		*
C*									*
C* Wilks, D.S., 2006:  STATISTICAL METHODS IN THE ATMOSPHERIC		*
C*    SCIENCES.  Academic Press, 627 pp.				*
C*									*
C*									*
C* PLT_ZSIG  ( MP, MT, NTRC, TRC, NPTS, PRMNAM, IRET )			*
C*									*
C* Input parameters:							*
C*	MP		INTEGER		Maximum points on a trace	*
C*	MT		INTEGER		Maximum number of traces	*
C*	NTRC		INTEGER		Number of traces to plot	*
C*	TRC (MP,MT)	REAL		Trace data			*
C*	NPTS (MT)	INTEGER		Number of trace points		*
C*	PRMNAM (MT)	CHAR*		Trace parameter labels		*
C*	CLBL1		CHAR*		Label for first trace point     *
C*	CLBL2		CHAR*		Label for last trace point	*
C*									*
C* Output parameters:							*
C*	IRET		INTEGER		Return code			*
C*				 	  0 = normal return 		*
C**									*
C* Log:									*
C* K. Brill/HPC      20061117   Created from PLT_STAT			*
C* K. Brill/HPC	     20070619	Check sercor to compute serial corr.	*
C************************************************************************
	INCLUDE		'plt.cmn'
C*
	PARAMETER	( RMISSD = -9999.  )
	PARAMETER	( RDIFFD =  0.1    )
	PARAMETER	( PI = 3.1415927 )
	PARAMETER	( MXPV = 5 )
C*
	CHARACTER*(*)   PRMNAM (mt), clbl1, clbl2
	REAL		trc (mp,mt)
	INTEGER		npts (*)
C*
	CHARACTER*40	pad
	REAL		pvals (MXPV)
	LOGICAL		ERMISS, dolag
	DOUBLE PRECISION
     +			xsum, x2sum, xsumm, xsump, rmn, rnmp, xx,
     +			rvar, rn, x2sup, xbarm, xbarp, rho
	DATA		pvals / .1, .05, .01, .005, .001 /
C*
	ERMISS (xxxx) = ( ABS ( xxxx - RMISSD ) .lt. RDIFFD )
C*-----------------------------------------------------------------------
	iret = 0
	dolag = .false.
	pad = ' '
C*
	IF ( ntrc .ne. 2 ) THEN
	    iret = -28
	    RETURN
	END IF
	IF ( npts(1) .ne. npts(2) ) THEN
	    iret = -29
	    RETURN
	END IF
	OPEN ( UNIT=61, FILE='fvs_sig_test1.txt', IOSTAT=ios,
     +         STATUS='UNKNOWN' )
	IF ( ios .ne. 0 ) THEN
	    iret = -30
	    RETURN
	ELSE
	    WRITE (6,*) ' '
	    WRITE (6,*) '  See file fvs_sig_test1.txt for output.'
	    WRITE (6,*) ' '
	END IF
C
C*	Write out the title for the output.
C
	WRITE (61,'(24X,A)') 'STATISTICAL SIGNIFICANCE ANALYSIS'
	WRITE (61,'(25X,A)')  'FOR PAIRED DATA FROM TWO TRACES'
C*
	xsum = 0.
	x2sum = 0.
        xsumm = 0.
        xsump = 0.
	nsumm = 0
	nsump = 0
	icnt = 0
	DO i = 1, npts(1)
	    IF ( .not. ERMISS ( trc(i,1) ) .and.
     +           .not. ERMISS ( trc(i,2) ) ) THEN
		icnt = icnt + 1
		xx = trc(i,1) - trc(i,2)
		xsum = xsum + xx
		x2sum = x2sum + xx * xx
		IF ( i .lt. npts(1) ) THEN
		    IF ( .not. ERMISS ( trc(i+1,1) ) .and.
     +			 .not. ERMISS ( trc(i+1,2) ) ) THEN
C
C*			If the point ahead is not missing, sum
C*			this to the "minus" sum.
C
		        xsumm = xsumm + xx
			nsumm = nsumm + 1
		    END IF
		END IF
		IF ( i .gt. 1 ) THEN
		    IF ( .not. ERMISS ( trc(i-1,1) ) .and.
     +     	         .not. ERMISS ( trc(i-1,2) ) ) THEN
C
C*			If the point behind is not missing, sum
C*			this to the "plus" sum.
C
		        xsump = xsump + xx
			nsump = nsump + 1
		    END IF
		END IF
	    END IF
	END DO
C*
	IF ( icnt .le. 3 ) THEN
	    DO it = 1, 2
	        WRITE (61,1004) it
	        CALL ST_LSTR ( trclbl (it), nchrs, ier )
	        CALL ST_LSTR ( prmnam (it), nchrp, ier )
		nc = nchrs + nchrp
	        ixoff = 40 - nc / 2
	        IF ( ixoff .gt. 0 ) THEN
		    IF ( INDEX ( trclbl(it)(1:nchrs),
     +				 prmnam(it)(1:nchrp) ) .eq. 0 ) THEN
	                WRITE (61,1002) pad (1:ixoff), 
     +	                        trclbl (it) (1:nchrs),
     +                          prmnam (it) (1:nchrp)
		    ELSE
			ixoff = 40 - nchrs / 2
	                WRITE (61,1002) pad (1:ixoff), 
     +	                        trclbl (it) (1:nchrs)
		    END IF
	        ELSE
		    IF ( INDEX ( trclbl(it)(1:nchrs),
     +		                 prmnam(it)(1:nchrp) ) .eq. 0 ) THEN
	                WRITE (61,1002) trclbl (it) (1:nchrs),
     +                               prmnam (it) (1:nchrp)
		    ELSE
	                WRITE (61,1002) trclbl (it) (1:nchrs)
		    END IF
	        END IF
	    END DO
	    WRITE (61,*) ' '
	    WRITE (61,*) ' THERE IS TOO LITTLE NON-MISSING TRACE DATA.'
	    WRITE (61,*) ' '
	    RETURN
	END IF
C*
        rn = FLOAT ( icnt )
	rmn = xsum / rn
	rvar = x2sum / rn - rmn * rmn
C*
	IF ( .not. sercor .or. nsumm .le. 3 .or. nsump .le. 3 ) THEN
C
C*	    Cannot compute lag-1 autocorrelation.
C
	    rho = 0.0
	ELSE
C
C*	    Compute lag-1 autocorrelation.
C
	    rnmp = FLOAT (nsumm)
	    xbarm = xsumm / rnmp
	    rnmp = FLOAT (nsump)
	    xbarp = xsump / rnmp
	    x2sum = 0.
	    x2sup = 0.
	    xsumm = 0.
	    DO i = 1, (npts(1) - 1)
		IF ( .not. ERMISS ( trc(i,1) ) .and.
     +		     .not. ERMISS ( trc(i,2) ) .and.
     +		     .not. ERMISS ( trc(i+1,1) ) .and.
     +		     .not. ERMISS ( trc(i+1,2) ) ) THEN
		    xx = trc(i,1) - trc(i,2) - xbarm
		    xxp = trc(i+1,1) - trc(i+1,2) - xbarp
		    xsumm = xsumm + xx * xxp
		    x2sum = x2sum + xx * xx
		    x2sup = x2sup + xxp * xxp
		END IF
	    END DO
	    rho = xsumm / DSQRT ( x2sum * x2sup )
	    IF ( rho .eq. 1.0 ) THEN
		rho = .95
	    ELSE IF ( rho .eq. -1.0 ) THEN
		rho = -.95
	    END IF
	    dolag = .true.
	END IF
C
C*	Compute sample variance estimate.
C
	zraw = rmn / DSQRT ( rvar / rn )
	rvar = ( rvar / rn ) * ( 1. + rho ) / ( 1. - rho )
	rsig = DSQRT ( rvar )
	zz = rmn / rsig
	zabs = ABS ( zz )
C
C*      Compute the probablility of z > zabs.
C
	CALL PLT_NCDF ( zabs, 0, 1.0, alpha, ier )
	ptail = 1.0 - alpha
C
C*	Write out title information.
C
	WRITE (61,*) ' '
	CALL ST_LSTR ( ptitle, lng, ier )
	itend = 0
	istrt = 1
	nclft = lng
	nchrx = 72
	DO WHILE ( itend .lt. lng )
	    IF ( nclft .gt. nchrx ) THEN
	        ic = itend + nchrx
	        DO WHILE ( ic .gt. 1 )
	            ic = ic - 1
	            IF ( ptitle (ic:ic) .eq. ' ' ) THEN
			isplit = ic
			ic = -9
		    END IF
		END DO
		IF ( ic .eq. -9 ) THEN
		    itend = isplit
		ELSE
		    itend = lng + 1
		END IF
	    ELSE
		itend = lng + 1
	    END IF
	    nchrs = itend - istrt
	    ixoff = 40 - nchrs / 2
	    IF ( ixoff .gt. 0 .and. ixoff .lt. 40 ) THEN
	        WRITE (61,1002) pad (1:ixoff),
     +	                       ptitle (istrt:itend-1)
	    ELSE
	        WRITE (61,1002) ptitle (istrt:itend-1)
	    END IF
1002	    FORMAT ( A, A, 1X, A )
	    istrt = itend + 1
	    nclft = lng - istrt + 1
	END DO
C
C*	Write out x-axis end point labels.
C
	CALL ST_LSTR ( clbl1, n1, ier )
	CALL ST_LSTR ( clbl2, n2, ier )
	nc = n1 + n2 + 4
	ixoff = 40 - nc / 2
	WRITE (61,*) ' '
	WRITE (61,1005) pad(1:ixoff), clbl1(1:n1), clbl2(1:n2)
1005	    FORMAT ( A, A, ' TO ', A )
C
C*	Write out trace labels.
C
	DO it = 1, ntrc
	    WRITE (61,1004) it
1004	    FORMAT ( /, 35X, 'TRACE # ', I2 )
	    CALL ST_LSTR ( trclbl (it), nchrs, ier )
	    CALL ST_LSTR ( prmnam (it), nchrp, ier )
	    nc = nchrs + nchrp
	    ixoff = 40 - nc / 2
	    IF ( ixoff .gt. 0 ) THEN
		IF ( INDEX ( trclbl(it)(1:nchrs), prmnam(it)(1:nchrp) )
     +		     .eq. 0 ) THEN
	            WRITE (61,1002) pad (1:ixoff), 
     +	                        trclbl (it) (1:nchrs),
     +                          prmnam (it) (1:nchrp)
		ELSE
		    ixoff = 40 - nchrs / 2
	            WRITE (61,1002) pad (1:ixoff), 
     +	                        trclbl (it) (1:nchrs)
		END IF
	    ELSE
		IF ( INDEX ( trclbl(it)(1:nchrs), prmnam(it)(1:nchrp) )
     +		     .eq. 0 ) THEN
	            WRITE (61,1002) trclbl (it) (1:nchrs),
     +                               prmnam (it) (1:nchrp)
		ELSE
	            WRITE (61,1002) trclbl (it) (1:nchrs)
		END IF
	    END IF
	END DO
C
C*	Write out the statistical significance analysis results.
C
	WRITE (61,*) ' '
	WRITE (61,*) 'Null Hypothesis (trace1 = trace2):'
	WRITE (61,*)
     +  '   The mean of the differences of the paired values from the'
	WRITE (61,*)
     +  '   two traces is zero.'
	WRITE (61,*) ' '
	IF ( rmn .lt. 0 ) THEN
	    WRITE (61,*) 'Alternative Hypothesis (trace1 < trace2):'
	    WRITE (61,*)
     +	    '   The mean of the paired differences is less than zero.'
	ELSE
	    WRITE (61,*) 'Alternative Hypothesis (trace1 > trace2):'
	    WRITE (61,*)
     +	    '   The mean of the paired differences exceeds zero.'
	END IF
	WRITE (61,*) ' '
	WRITE (61,'(A,2X,F9.6)')
     +  ' Gaussian probability of wrongly rejecting null hypothesis:',
     +   ptail
	WRITE (61,*) ' '
	WRITE (61,* )
     +  '        Test Level   Decision (one-sided Gaussian) '
	DO i = 1, MXPV
	    IF ( pvals(i) .ge. ptail ) THEN
		pad = 'REJECT null hypothesis'
		nch = 22
	    ELSE
		pad = 'ACCEPT null hypothesis'
		nch = 22
	    END IF
	    WRITE (61,1001) pvals(i), pad(1:nch)
1001	    FORMAT (11X,F5.3,7X,A)
	END DO
	WRITE (61,*) ' '
	WRITE (61,1003) zraw, zz, icnt, rho
1003	FORMAT (' ZRAW = ',1PE13.5,'  ZTST = ',E13.5,'  N = ', I8,
     +          '  Lag1 Corr = ', 0PF9.6 )
	IF ( .not. dolag .and. sercor ) THEN
	    WRITE (61,*) ' '
	    WRITE (61,*) 'Serial correlation could not be computed.'
	    WRITE (61,*) 'Serial correlation is assumed to be zero.'
	ELSE IF ( .not. sercor ) THEN
	    WRITE (61,*) ' '
	    WRITE (61,*) 'The user set serial correlation to zero.'
	END IF
	CLOSE (UNIT=61)
C*
	RETURN
	END
