	SUBROUTINE PHASERR ( af, bf, ao, bo, km, numwv, degerr, kmxerr,
     +			     varcnf, varcno, paf, pao, iret )
C************************************************************************
C* PHASERR								*
C*									*
C* This subroutine computes the phase error for a specified number of	*
C* wave number components.						*
C*									*
C* The variance contributions of the forecast and observations are	*
C* sorted in descending order of magnitude.  The wave number		*
C* associated with each contribution must be the same for both the	*
C* forecast and observations for each of the output phase error		*
C* elements.  If this is not the case, then the forecast and		*
C* observations have too different a spectral composition for a phase	*
C* error calculation.  Only the first NUMWV most significant spectral	*
C* components are examined.  The actual number of spectral components	*
C* compared is returned in NUMWV.					*
C* 									*
C* The phase error and variance contribution are computed only for each	*
C* one of the NUMWV most significant matching wave number components.	*
C*									*
C*									*
C* PHASERR  ( AF, BF, AO, BO, KM, NUMWV, DEGERR, KMXERR, VARCNF,	*
C*	      VARCNO, PAF, PAO, IRET )					*
C*									*
C* Input parameters:							*
C*	AF (KM)		REAL*8		Forecast COSINE coefficients	*
C*	BF (KM)		REAL*8		Forecast SINE coefficients	*
C*	AO (KM)		REAL*8		Observation COSINE coefficients	*
C*	BO (KM)		REAL*8		Observation SINE coefficients	*
C*	KM		INTEGER		Number of coefficients		*
C*									*
C* Input and output parameters:						*
C*	NUMWV		INTEGER		Max # of components requested	*
C*					Output # of matching components *
C*									*
C* Output parameters:							*
C*	DEGERR (NUMWV)	REAL		Phase error (degrees)		*
C*	KMXERR (NUMWV)	INTEGER		Wave number component of error	*
C*	VARCNF (NUMWV)	REAL		Frcst variance contributions	*
C*	VARCNO (NUMWV)	REAL		Obs variance contributions	*
C*	PAF (NUMWV)	REAL		Frcst phase angle (deg)		*
C*	PAO (NUMWV)	REAL		Obs phase angle	 (deg)		*
C*	IRET		INTEGER		Return code			*
C*					  0 = normal return		*
C*					 -3 = KM is too large		*
C*					 -4 = no phase error calculation*
C**									*
C* Log:									*
C* K. Brill/HPC		 4/00						*
C* K. Brill/HPC		 6/00	Do matched components <= NUMWV		*
C* K. Brill/HPC		 6/00	Reworked using phase ang in COS series  *
C*				and return PAF and PAO			*
C* K. Brill/HPC		 7/00	Double precision arrays			*
C************************************************************************
	INCLUDE		'GEMPRM.PRM'
	PARAMETER	( MXC = 256 )
C*
	DOUBLE
     +	PRECISION	af (*), bf (*), ao (*), bo (*)
	REAL		degerr (*)
	REAL		varcnf (*), varcno (*), paf (*), pao (*)
	INTEGER		kmxerr (*)
C*
	DOUBLE
     +	PRECISION 	cf (MXC), co (MXC)
	INTEGER		kpt (MXC,2)
	LOGICAL		ok
C------------------------------------------------------------------------
	iret = 0
	IF ( km .gt. MXC ) THEN
	    iret = -3
	    RETURN
	END IF
C
C*	Compute the component variance contribution C values.
C
	DO i = 1, km
	    cf (i) = af (i) * af (i) + bf (i) * bf (i)
	    co (i) = ao (i) * ao (i) + bo (i) * bo (i)
	    kpt (i,1) = i
	    kpt (i,2) = i
	END DO
C
C*	Perform a sort of the C values.
C
	DO i = 1, km-1
	    DO j = i+1, km
		IF ( cf (kpt(i,1)) .lt. cf (kpt(j,1)) ) THEN
		    ksave = kpt(i,1)
		    kpt(i,1) = kpt(j,1)
		    kpt(j,1) = ksave
		END IF
		IF ( co (kpt(i,2)) .lt. co (kpt(j,2)) ) THEN
		    ksave = kpt(i,2)
		    kpt(i,2) = kpt(j,2)
		    kpt(j,2) = ksave
		END IF
	    END DO
	END DO
C
C*	Check to get the number of the wave number components
C*	that are equal for the first NUMWV sorted values.
C
	ok = .true.
	i = 0
	icnt = 0
	DO WHILE ( ok .and. i .lt. numwv )
	    i = i + 1
	    ok = ( kpt (i,1) .eq. kpt (i,2) )
	    IF ( ok ) icnt = icnt + 1
	END DO
	numwv = icnt
	IF ( numwv .eq. 0 ) THEN
	    iret = -4
	    RETURN
	END IF
C
C*	Compute the phase errors in absolute degrees.
C
	DO i =1, numwv
	    j = kpt (i,1)
	    varcnf (i) = cf (j) / 2.
	    varcno (i) = co (j) / 2.
	    scf = DSQRT ( cf (j) )
	    sco = DSQRT ( co (j) )
	    cosf = af (j) / scf
	    sinf = bf (j) / scf
	    coso = ao (j) / sco
	    sino = bo (j) / sco
	    phif = ATAN2 ( sinf, cosf )
	    phio = ATAN2 ( sino, coso )
	    paf (i) = phif * RTD
	    pao (i) = phio * RTD
	    phi = ( paf (i) - pao (i) )
	    IF ( phi .gt. 180.0 ) phi = phi - 360.
	    IF ( phi .lt. -180.0 ) phi = phi + 360.
	    degerr (i) = phi
	    kmxerr (i) = j
	END DO
C*
	RETURN
	END
