        SUBROUTINE GDLINT ( inttyp, rnvblk, rnavo, grid, grdo, iret )
C************************************************************************
C* GDLINT								*
C*									*
C* This subroutine interplates the data in the input grid to the	*
C* output grid whose navigation is given by RNAVO.			*
C*									*
C* GRID and GRDO may be the same array.					*
C*									*
C* The following procedure is applied to the output grid points:	*
C*									*
C*   1-  The output grid navigation is set in GPLT using GR_SNAV.	*
C*   2-  The lat/lons of output grid points are computed using GTRANS.	*
C*   3-  The input navigation is reset in GPLT by calling GR_SNAV.	* 
C*   4-  The lat/lons computed in 3 are transformed to (I,J) relative	*
C*       to the input grid using GTRANS.				*
C*   5-  The input grid values are interpolated to the out grid using	*
C*       GR_INTP.							*
C*									*
C* GDLINT ( INTTYP, RNVBLK, RNAVO, GRID, GRDO, IRET )			*
C*									*
C* Input parameters:							*
C*	INTTYP		INTEGER		Interpolation type		*
C*					  0 = linear interpolation	*
C*					  1 = nearest point assignment	*
C*	RNVBLK (LLNNAV) REAL		Navigation block of GRID	*
C*	RNAVO (LLNNAV)	REAL		Navigation block for output grid*
C*	GRID (*)	REAL		Grid of data			*
C*									*
C* Output parameters:							*
C*	GRDO (*)	REAL		Output grid of interplated data *
C*	IRET		INTEGER		Return code			*
C*					  0 = normal return		*
C*					 -7 = invalid nav info		*
C*					 -8 = invalid grid size		*
C*					 -9 = invalid interp type	*
C**									*
C* Log:									*
C* K. Brill/HPC		 1/00						*
C************************************************************************
	INCLUDE		'GEMPRM.PRM'
	REAL 		grid (*), grdo (*), rnavo (*), rnvblk (*)
C*
	LOGICAL		same
	REAL		glat (LLMXGD), glon (LLMXGD), gout (LLMXGD)
	REAL		rnvsi (LLNNAV), rnvso (LLNNAV)
	LOGICAL		allset
C*
	SAVE		glat, glon, rnvsi, rnvso, allset, np
	DATA		allset / .false. /
C*
	INCLUDE		'ERMISS.FNC'
	INDXQ ( lx, ly, kx ) = ( ly - 1 ) * kx + lx
C------------------------------------------------------------------------
	iret = 0
C
C*	Initialize saved navigation information.
C
	IF ( .not. allset ) THEN
	    allset = .true.
	    DO i = 1, LLNNAV
		rnvsi (i) = rnvblk (i)
		rnvso (i) = rnavo (i)
	    END DO
	    same = .false.
	ELSE
	    CALL GR_CNAV ( rnvblk, rnvsi, 13, same, ier )
	    IF ( same ) CALL GR_CNAV ( rnavo, rnvso, 13, same, ier )
	END IF
	maxx = rnvblk (5)
	maxy = rnvblk (6)
C
C*	Initialize all grid points to missing value.
C
	nxo = NINT ( rnavo (5) )
	nyo = NINT ( rnavo (6) )
	npg = nxo * nyo
	DO i = 1, npg
	    gout (i) = RMISSD
	END DO
C
C*	If the navigations have changed, recompute GLAT and GLON.
C
	IF ( .not. same ) THEN
	    np = nxo * nyo
	    IF ( np .gt. LLMXGD ) THEN
		iret = -8
		RETURN
	    END IF
	    DO i = 1, LLNNAV
		rnvsi (i) = rnvblk (i)
		rnvso (i) = rnavo (i)
	    END DO
C
C*	    Compute (i,j) of each output grid point.
C
	    jj = 0
	    DO j = 1, nyo
		DO i = 1, nxo
		    jj = jj + 1
		    glat (jj) = FLOAT (i)
		    glon (jj) = FLOAT (j)
		END DO
	    END DO
C
C*	    Set navigation of output grid in GPLT.
C
	    CALL GR_SNAV ( 16, rnavo, ier )
	    IF ( ier .ne. 0 ) THEN
		CALL ER_WMSG ( 'GR', ier, ' ', ier2 )
		iret = -7
		RETURN
	    END IF
C
C*	    Transform output grid coords to lat/lon.
C
	    CALL GTRANS ( 'G', 'M', jj, glat, glon, glat, glon,
     +		          ier )
	    IF ( ier .ne. 0 ) THEN
		iret = -7
		RETURN
	    END IF
C
C*	    Set navigation of input grid in GPLT.
C
	    CALL GR_SNAV ( 16, rnvblk, ier )
	    IF ( ier .ne. 0 ) THEN
		CALL ER_WMSG ( 'GR', ier, ' ', ier2 )
		iret = -7
		RETURN
	    END IF
C
C*	    Transform lat/lon to input grid coords.
C
	    CALL GTRANS ( 'M', 'G', jj, glat, glon, glat, glon,
     +			      ier )
	    IF ( ier .ne. 0 ) THEN
		iret = -7
		RETURN
	    END IF
C
C*	    Eliminate invalid interpolation points.
C
	    DO i = 1, jj
		IF ( ERMISS ( glat (i) ) .or. ERMISS ( glon (i) )
     +		     .or. glat (i) .lt. 1 .or. glon (i) .lt. 1
     +		     .or. glat (i) .gt. maxx .or. glon (i) .gt. maxy )
     +		THEN
		    glat (i) = RMISSD
		    glon (i) = RMISSD
		END IF
	    END DO
	END IF
C
C*	Interpolate to output grid.
C
	IF ( inttyp .eq. 0 ) THEN
C
C*	    Bilinear interpolation.  Note:  GR_INTP is not used
C*	    because it does no check for missing coordinates.
C
	    DO ii = 1, np
		IF ( .not. ERMISS ( glat (ii) ) ) THEN
		    icol = glat (ii)
		    irow = glon (ii)
		    IF ( icol .eq. maxx ) icol = icol - 1
		    IF ( irow .eq. maxy ) irow = irow - 1
		    icolp1 = icol + 1
		    irowp1 = irow + 1
		    c = glat (ii) - FLOAT (icol)
		    r = glon (ii) - FLOAT (irow)
		    omc = 1. - c
		    omr = 1. - r
		    IF ( ERMISS (grid(INDXQ(icol,irow,maxx)))
     +			 .or.
     +			 ERMISS (grid(INDXQ(icolp1,irow,maxx)))
     +			 .or.
     +			 ERMISS (grid(INDXQ(icol,irowp1,maxx)))
     +			 .or.
     +			 ERMISS (grid(INDXQ(icolp1,irowp1,maxx))) )
     +		    THEN
			gout (ii) = RMISSD
		    ELSE
			gout (ii) =
     +			( grid (INDXQ(icol,irow,maxx)) * omc +
     +			  grid (INDXQ(icolp1,irow,maxx)) * c ) * omr +
     +		        ( grid (INDXQ(icol,irowp1,maxx)) * omc +
     +		          grid (INDXQ(icolp1,irowp1,maxx) ) * c ) * r
		    END IF
		END IF
	    END DO
	ELSE IF ( inttyp .eq. 1 ) THEN
C
C*	    Nearest value interpolation (NVI).
C
	    DO ii = 1, np
		IF ( .not. ERMISS ( glat (ii) ) ) THEN
		    nrx = NINT ( glat (ii) )
		    nry = NINT ( glon (ii) )
		    gout (ii) = grid (INDXQ(nrx,nry,maxx))
		END IF
	    END DO
	ELSE
	    iret = -9
	    RETURN
	END IF
	DO ii = 1, np
	    grdo (ii) = gout (ii)
	END DO
C*
	RETURN
	END
