	SUBROUTINE RDTD_IRP (  itrc, ndd, iret )
C************************************************************************
C* RDTD_IRP								*
C*									*
C* This subroutine initializes the percentile levels needed to compute  *
C* the Hersbach Continuous Ranked Probability Score (CRPS) and its      *
C* reliability component.  The statistic type is CRPS.                  *
C*									*
C* The number of percentile level values required for the calibration   *
C* is N = (NDD - 6) / 2.  The number N may or may not appear following  *
C* a slash after "CRPS" in the statistic type field.                    *
C*									*
C* If the calibration for the percentile levels cannot be read from the	*
C* user file (crps_N.prob, where N is the number of percentile levels), *
C* then they are set to a default calibration if N=7, 9, or 11.  If N   *
C* is not equal to either 7, 9, or 11 and no user file exists, a fatal  *
C* error occurs.                                                        *
C*									*
C* Below is an exmple file for N=5.                                     *
C*									*
C*        PERCENTILE LEVEL      CUMULATIVE PROBABILITY                  * 
C*        LEVEL_1               0.10                                    *
C*        LEVEL_2               0.25                                    *
C*        LEVEL_3               0.50                                    *
C*        LEVEL_4               0.75                                    *
C*        LEVEL_5               0.90                                    *
C*									*
C* Each data record following the header has a label string without	*
C* embedded blanks and a numerical probability value.  The records are	*
C* ordered from lowest to highest cumulative probability.               *
C*									*
C* RDTD_IRP ( ITRC, NDD, IRET )  					* 
C*									*
C* Input parameters:							*
C*	ITRC		INTEGER		Trace number			*
C*      NDD		INTEGER		The trace data depth		*
C*									*
C* Output parameters:							*
C*	IRET		INTEGER		Return code			*
C*					  0 = Normal return		*
C*					-64 = too many cumulative probs *
C*					-65 = too many probs in file    *
C*                                      -66 = too few cumulative probs  *
C*									*
C* Log:									*
C* K. Brill/IMSG      20160629  Began by modifying RDTD_IPB             *
C* K. Brill/IMSG      20160718  Define default calibration for N = 11   *
C************************************************************************
	INCLUDE			'rdtd.cmn'
C*
C*
        CHARACTER*2             cncrp
	CHARACTER*72		recrd
	CHARACTER*48		errmsg
	CHARACTER*24		carry (2)
	CHARACTER*32		filnam
	LOGICAL			exist
C------------------------------------------------------------------------
	iret = 0
C*
	ncprb = ( ndd - 6 ) / 2
	IF ( ncprb .gt. MXNCRP ) THEN
	    iret = -64
	    RETURN
	END IF
        CALL ST_INCH ( ndd, cncrp, ier )
	CALL ST_LSTR ( cncrp, lng, ier )
	filnam = 'crps_' // cncrp (1:lng) // '.prob'
	icnt = 0
	INQUIRE ( FILE = filnam, EXIST = exist, IOSTAT = ios )
	IF ( exist ) THEN
	    OPEN ( FILE = filnam, UNIT = 99, IOSTAT = ios )
	    IF ( ios .ne. 0 ) THEN
		iret = 3
		errmsg = filnam // ' not readable -- defaults used'
		CALL RDTD_ERR ( iret, errmsg, ier )
	    ELSE
C
C*		Skip past initial label record.
C
		READ ( 99, '(A)', IOSTAT = ios ) recrd
C*
		DO WHILE ( ios .eq. 0 )
		    READ ( 99, '(A)', IOSTAT = ios ) recrd
		    IF ( ios .eq. 0 ) THEN
			CALL ST_CLST ( recrd, ' ', ' ', 2,
     +				       carry, num, ier )
			IF ( ier .eq. 0 ) THEN
			    CALL ST_CRNM ( carry(2), xx, ier )
			    IF ( ier .eq. 0 .and. xx .le. 1.
     +				 .and. xx .ge. 0. ) THEN
				icnt = icnt + 1
				IF ( icnt .gt. ncprb ) THEN
				    iret = -65
				    RETURN
				END IF
				crpspl ( icnt, itrc ) = xx
				ncrpsp (itrc) = icnt
			    END IF
			END IF
		    END IF
		END DO
	    END IF
	    CLOSE ( UNIT = 99 )
	END IF
        IF ( icnt .lt. ncprb ) THEN
            WRITE (6,*) ' Too few or no user defined cumulative ',
     +                  'probs -- using default values.'
        ELSE
            WRITE (6,*) ' Applying user defined cumulative probs.'
        END IF
C
C*	Add new default discretizations here.  Do not exceed MXNCRP.
C
	IF ( ncprb .eq. 7 ) THEN
	    IF ( icnt .lt. 7 ) THEN
		crpspl (1,itrc) = .05
		crpspl (2,itrc) = .10
		crpspl (3,itrc) = .25
		crpspl (4,itrc) = .50
		crpspl (5,itrc) = .75
		crpspl (6,itrc) = .90
		crpspl (7,itrc) = .95
		ncrpsp (itrc) = 7
            END IF
        ELSE IF ( ncprb .eq. 9 ) THEN
	    IF ( icnt .lt. 9 ) THEN
		crpspl (1,itrc) = .05
		crpspl (2,itrc) = .10
		crpspl (3,itrc) = .25
		crpspl (4,itrc) = .40
		crpspl (5,itrc) = .50
		crpspl (6,itrc) = .60
		crpspl (7,itrc) = .75
		crpspl (8,itrc) = .90
		crpspl (9,itrc) = .95
		ncrpsp (itrc) = 9
            END IF
        ELSE IF ( ncprb .eq. 11 ) THEN
	    IF ( icnt .lt. 11 ) THEN
		crpspl (1,itrc) = .05
		crpspl (2,itrc) = .10
		crpspl (3,itrc) = .15
		crpspl (4,itrc) = .25
		crpspl (5,itrc) = .40
		crpspl (6,itrc) = .50
		crpspl (7,itrc) = .60
		crpspl (8,itrc) = .75
		crpspl (9,itrc) = .85
		crpspl (10,itrc) = .90
		crpspl (11,itrc) = .95
		ncrpsp (itrc) = 11
            END IF
        ELSE IF ( icnt .lt. ncprb ) THEN
            iret = -66
            RETURN
        END IF
C*
        WRITE (6,*)
     +       ' Cumulative probabilities for trace # ', itrc, ': '
        DO i = 1, ncrpsp (itrc)
            WRITE (6,1000) i, crpspl (i,itrc)
1000        FORMAT ( ' Level ', I3, ' cumulative probability = ',
     +		    F5.2 )
        END DO
C*
	RETURN
	END
