	SUBROUTINE RSMP_DRV ( iplt, iutrc, nutrc, iuccd, nuccd,
     +			      ixp, alpha, rmiss, mxt,
     +			      sbxls, smxs, smns, smdn, skew, nss, iret )
C************************************************************************
C* RSMP_DRV								*
C*									*
C* This subroutine performs resampling for a set of traces and compu-	*
C* tational codes.  The resampling is done for all the unique trace	*
C* pairs for all computational codes at a given point along the trace.	*
C* The values are returned in arrays with the trace pairs incrementing	*
C* most rapidly and the computational codes most slowly. So, all of the	*
C  half-box lengths and whisker extensions for the maximum and minimum	*
C* differences are returned for all the trace pairs for the first	*
C* computational code, followed by the second, and so on.		*
C*									*
C* RSMP_DRV ( IPLT, IUTRC, NUTRC, IUCCD, NUCCD, IXP, ALPHA,		*
C*	      RMISS, MXT, SBXLS, SMXS, SMNS, SMDN, SKEW, NSS, IRET )	*
C*									*
C* Input parameters:							*
C*	IPLT		INTEGER		Plot number in trace.ucd file	*
C*	IUTRC		INTEGER		Unique trace numbers to locate  *
C*					traces in trace.ucd file	*
C*	NUTRC		INTEGER		Number of unique trace numbers  *
C*	IUCCD		INTEGER		Unique computational codes to	*
C*					apply to all traces		*
C*	NUCCD		INTEGER		Number of unique comp codes	*
C*	IXP		INTEGER		Number of trace point (x index)	*
C*	ALPHA		REAL		Significance level probability  *
C*	RMISS		REAL		Missing value			*
C*      MXT             INTEGER         MXTRCS value for dimension      *
C*									*
C* Output parameters:							*
C*	SBXLS (MXT,2)	REAL		Lower, upper boxlengths         *
C*	SMXS  (MXTRCS)	REAL		Whisker extension for maxima	*
C*	SMNS  (MXTRCS)	REAL		WHISKER extension for minima	*
C*      SMDN  (MXTRCS)  REAL            Median of resampled differences *
C*      SKEW  (MXTRCS)  REAL            Skew ratio of differences       *
C*	NSS		INTEGER		Number of objects returned	*
C*	IRET		INTEGER		Return code			*
C*				 	  0 = normal return 		*
C*					 -1 = cannot open trace.ucd	*
C**									*
C* Log:									*
C* K. Brill/HPC      20120321	Dummy output				*
C* K. Brill/HPC      20120330	Completed				*
C* K. Brill/HPC      20120416   Let user set number of trials		*
C* K. Brill/HPC      20120420   Edit nsmpls warning message		*
C* K. Brill/HPC      20120912   Continue if RSMP_CAL iret > 0		*
C* K. Brill/WPC      20140825   Return SMDN = median differences        *
C* K. Brill/WPC      20140919   Return SKEW = above/below median ratio; *
C*                              sbxls returns below/above median values *
C*                              as symmetry about median is not assumed *
C* K. Brill/WPC      20140927   Return box limits as the percentile     *
C*                              difference values about zero            *
C* K. Brill/WPC      20141104   Set skew to missing if division by zero *
C************************************************************************
	INCLUDE		'rsmp.cmn'
C*
	INTEGER		iutrc (*), iuccd (*)
	REAL		sbxls (MXT,2), smxs (*), smns (*), smdn (*),
     +                  skew (*)
C*
	LOGICAL		stor
	REAL*8		recm (MXDPTH), sumd, sumd2, rcnt
	REAL		prctl (3)
C*------------------------------------------------------------------------
	iret = 0
C
C*	Initialize:
C
	IF ( nsmpls .lt. 100 .or. nsmpls .gt. 10004 ) THEN
	    WRITE (6,*)  'Warning:  Number of trials reset to 2000.'
	    nsmpls = 2000
	END IF
	lunucd = 21
	DO i = 1, MXTRCS
	    nucdrs (i) = 0
	    lmissd (i) = .true.
	    DO j = 1, MXDPTH
		DO k = 1, MXRSMP
		    prdsmp (1,i,j,k) = 0.
		    prdsmp (2,i,j,k) = 0.
		END DO
	    END DO
	END DO
C
C*	Open the input file, trace.ucd.
C
	OPEN (UNIT=lunucd, FILE='trace.ucd', STATUS='OLD',
     +	      ACCESS='SEQUENTIAL', FORM='UNFORMATTED',
     +	      IOSTAT=ios)
	IF ( ios .ne. 0 ) THEN
	    iret = -1
	    RETURN
	ELSE
C
C*	    Read the data depth value.
C
	    READ (lunucd,IOSTAT=ios) mdpth
	    IF ( ios .ne. 0 ) THEN
	        iret = -2
		CLOSE (UNIT=lunucd)
	        RETURN
	    END IF
	END IF
C*
	WRITE (6,*) ' '
	WRITE (6,*)
     +  ' Resampling at abscissa point # ',ixp,' with ',nsmpls,
     +  ' trials.'
C
C*	Fill the resampling uncombined data array.
C
	nrecr = 0
	DO WHILE ( ios .eq. 0 )
	    READ (lunucd,IOSTAT=ios) mplt, mtrc, mxpt,
     +			(recm(jj),jj=1,mdpth)
	    IF ( ios .eq. 0 ) THEN
		nrecr = nrecr + 1
		IF ( mplt .eq. iplt .and. mxpt .eq. ixp ) THEN
		    stor = .false.
		    it = 0
		    DO WHILE ( .not. stor .and. it .lt. nutrc )
			it = it + 1
			IF ( mtrc .eq. iutrc (it) ) THEN
			    stor = .true.
			    nucdrs (it) = nucdrs (it) + 1
			    IF ( nucdrs (it) .gt. MXRECS ) THEN
				iret = -3
				CLOSE (UNIT=lunucd)
				RETURN
			    END IF
			    DO id = 1, mdpth
				pntdta (it,id,nucdrs(it)) = recm (id)
			    END DO
			END IF
		    END DO
		END IF
	    END IF
	END DO
C*
	CLOSE (UNIT=lunucd)
C
C*	Check for record count parity among traces at this point.
C
	nrc = nucdrs (1)
	DO it = 2, nutrc
	    IF ( nrc .ne. nucdrs (it) ) THEN
		iret = -4
		RETURN
	    END IF
	END DO
	IF ( nrc .lt. 28 ) THEN
	    WRITE (6,*) ' WARNING -- Only ', nrc, ' records found for',
     +			' resampling at X point ', ixp, '.'
	END IF
C
C*	Do resampling.
C*	Note:  Accumulating all samples in memory reduces the number
C*	       of subroutine calls for calculating metrics.
C
	nprs = nutrc - 1
	ibmt = 1
	DO is = 1, nsmpls
	    DO ir = 1, nrc
		xran = rand ()
	        ipr = 0
	        DO it = 2, nutrc
		    ipr = ipr + 1
		    IF ( xran .lt. .5 ) THEN
		        it1 = ibmt
		        it2 = it
		    ELSE
		        it1 = it
		        it2 = ibmt
		    END IF
		    DO id = 1, mdpth
			IF ( pntdta (it1,id,ir) .gt. -1.1E30 .and.
     +			     pntdta (it2,id,ir) .gt. -1.1E30 ) THEN
     			    prdsmp (1,ipr,id,is) = prdsmp (1,ipr,id,is)
     +					         + pntdta (it1,id,ir)
			    prdsmp (2,ipr,id,is) = prdsmp (2,ipr,id,is)
     +					         + pntdta (it2,id,ir)
			    lmissd (ipr) = .false.
			END IF
		    END DO
		END DO
	    END DO
	END DO
C*
	ixo = 0
	DO icc = 1, nuccd
C*
	    DO ipr = 1, nprs
		ixo = ixo + 1
		IF ( lmissd (ipr) ) THEN
C
C*		    Set output arrays to missing values
C
		    sbxls (ixo,1) = rmiss
		    sbxls (ixo,2) = rmiss
		    smxs (ixo) = rmiss
		    smns (ixo) = rmiss
		    smdn (ixo) = rmiss
		    skew (ixo) = rmiss
C*
		ELSE
C
C*	            Build the array of samples to send to the metric
C*		    calculator.
C
		    DO is = 1, nsmpls
		        DO id = 1, mdpth
		            dtrace (1,id,is) = prdsmp(1,ipr,id,is)
		            dtrace (2,id,is) = prdsmp(2,ipr,id,is)
			END DO
		    END DO
C
C*		    CALL calculator for first member of pair:
C
		    CALL RSMP_CAL ( 1, iuccd (icc), rmiss, nsmpls, mdpth,
     +				    score1, xmin1, xmax1, iret )
		    IF ( iret .lt. 0 ) THEN
			CALL RSMP_ERR ( iret, 'Pair Member #1', irr )
			RETURN
		    ELSE IF ( iret .gt. 0 ) THEN
			CALL RSMP_ERR ( iret, 'Pair Member #1', irr )
			iret = 0
		    END IF
C
C*		    CALL calculator for second member of pair:
C
		    CALL RSMP_CAL ( 2, iuccd (icc), rmiss, nsmpls, mdpth,
     +				    score2, xmin2, xmax2, iret )
		    IF ( iret .lt. 0 ) THEN
			CALL RSMP_ERR ( iret, 'Pair Member #2', irr )
			RETURN
		    ELSE IF ( iret .gt. 0 ) THEN
			CALL RSMP_ERR ( iret, 'Pair Member #2', irr )
			iret = 0
		    END IF
C
C*		    Compute differences and extrema.
C
		    icnt = 0
		    DO is = 1, nsmpls
			IF ( ABS (score1 (is) - rmiss) .gt. .0005 .and.
     +			     ABS (score2 (is) - rmiss) .gt. .0005 )
     +			THEN
			    icnt = icnt + 1
			    dif = score1 (is) - score2 (is)
			    score1 (icnt) = dif
			END IF
		    END DO
		    IF ( icnt .gt. 0 ) THEN
		        nprc = 3
			wplvl = alpha / 2.
	        	prctl (1) = 1 - wplvl
	        	prctl (2) = wplvl
	        	prctl (3) = .5
	    	        CALL RSMP_SRT ( score1, icnt, rmiss, nprc, prctl,
     +			                score2, dmin, dmax, ier )
			IF ( ier .eq. 0 ) THEN
C
C*                          Above and below zero box lengths are
C*                          returned as positive values.
C
			    sbxls (ixo,2) = prctl (1)
			    sbxls (ixo,1) = - prctl (2)
                            IF ( prctl (2) .gt. 0.0 ) THEN
                                WRITE (6,*) 'RE-SAMPLING error--',
     +                          'right skewed difference statistics.'
                                 sbxls (ixo,1) = 0.0
                            END IF
C*
			    smxs (ixo) = dmax - prctl (1)
			    smns (ixo) = prctl (2) - dmin
                            smdn (ixo) = prctl (3)
                            dnom = prctl (3) - prctl (2)
                            IF ( dnom .gt. 0 ) THEN
                                skew (ixo) = ( prctl (1) - prctl (3) )
     +                                       / dnom
                            ELSE
                                skew (ixo) = rmiss
                            END IF
			ELSE
		            sbxls (ixo,1) = rmiss
		            sbxls (ixo,2) = rmiss
		            smxs (ixo) = rmiss
		            smns (ixo) = rmiss
		            smdn (ixo) = rmiss
                            skew (ixo) = rmiss
			END IF
		    ELSE
		        sbxls (ixo,1) = rmiss
		        sbxls (ixo,2) = rmiss
		        smxs (ixo) = rmiss
		        smns (ixo) = rmiss
		        smdn (ixo) = rmiss
                        skew (ixo) = rmiss
		    END IF
		END IF
	    END DO
	END DO
	nss = ixo
C*
	RETURN
	END
